@@ -3,8 +3,10 @@ import spacy
 import huspacy
 import benepar
 from enum import Enum, auto
-from typing import Union
+import nltk
 from nltk import Tree
+from typing import List, Dict, Set, Union
+from pathlib import Path
 from .export import export_figure
 
 
@@ -104,7 +106,7 @@ class BracketedTree:
         return bracketed_tree_string
 
     @staticmethod
-    def parentheses_locations(bracketed_tree_string: str) -> dict[int, int]:
+    def parentheses_locations(bracketed_tree_string: str) -> Dict[int, int]:
         """Validates that for each opening parenthesis in the given bracketed tree string there is a corresponding
         closing parenthesis. If this is not the case appropriate exceptions are raised.
 
@@ -275,7 +277,7 @@ class ConstituentTree:
     @staticmethod
     def create_pipeline(language: Language = Language.English, spacy_model_size: SpacyModelSize = SpacyModelSize.Small,
                         benepar_english_model: BeneparEnglishModel = BeneparEnglishModel.EN3,
-                        download_models: bool = False) -> spacy.Language:
+                        download_models: bool = True) -> spacy.Language:
         """Constructs the fundamental nlp pipeline for the given language that consists of a spaCy pipeline that
         incorporates the benepar component.
 
@@ -308,11 +310,18 @@ class ConstituentTree:
         """
 
         def download(spacy_model_id: str, benepar_model_id: str) -> None:
+            download_dir = Path(__file__).parent / Path(".resources")
+            download_dir.mkdir(parents=True, exist_ok=True)
+            if not str(download_dir) in nltk.data.path:
+                nltk.data.path.append(str(download_dir))
             if language == ConstituentTree.Language.Hungarian:
                 huspacy.download(spacy_model_id)
             else:
-                spacy.cli.download(spacy_model_id)
-            benepar.download(benepar_model_id)
+                if not spacy.util.is_package(spacy_model_id):
+                    print(f"'{spacy_model_id}' not found. Downloading...")
+                    spacy.cli.download(spacy_model_id, False, False, "--quiet")
+            benepar.download(benepar_model_id, download_dir=download_dir, quiet=True)
+
 
         def err_models_not_downloaded(model_id: str, framework: str, error_trace: str) -> str:
             return f'It seems that the specified model "{model_id}" has not been downloaded or installed yet.\n' \
@@ -451,7 +460,7 @@ class ConstituentTree:
         except LookupError as e:
             print(err_models_not_downloaded(model_id=benepar_model, framework='benepar', error_trace=e))
 
-    def _extract_phrases(self, tree: Tree, phrasal_category: str, min_words_in_phrases: int = 2) -> list[list]:
+    def _extract_phrases(self, tree: Tree, phrasal_category: str, min_words_in_phrases: int = 2) -> List[list]:
         """Extracts phrases according to a given phrasal category from an nltk.Tree in a recursive manner.
 
         Args:
@@ -504,7 +513,7 @@ class ConstituentTree:
         elif content_type == self.NodeContent.Combined:
             return ' '.join([f'{w[0]}_{w[1]}' for w in tree.pos()])
 
-    def extract_all_phrasal_categories(self) -> set[str]:
+    def extract_all_phrasal_categories(self) -> Set[str]:
         """Extracts all available phrasal categories in the tree.
 
         Returns:
@@ -514,7 +523,7 @@ class ConstituentTree:
         return set([str(p.lhs()) for p in self.nltk_tree.productions() if p.is_nonlexical() and p.lhs()])
 
     def extract_all_phrases(self, min_words_in_phrases: int = 2, avoid_nested_phrases: bool = False,
-                            content: NodeContent = NodeContent.Text) -> dict[str, list[str]]:
+                            content: NodeContent = NodeContent.Text) -> Dict[str, List[str]]:
         """Extracts all phrases from the tree and the categories they belong to.
 
         Args:
